/*
 * Test renaming operations.
 *
 * Copyright (C) 2020, 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <fsclient/client.h>
#include <systypes/fcntl.h>



/* Show a directory's contents and determine if all expected files are
   present. */

static int show_directory(const char *filename, int expected)
{
  file_t *reader = client_opendir(filename);

  if (!client_opened(reader))
  {
    client_close(reader);
    printf("Could not read from directory.\n");
    return 1;
  }

  printf("Reading...\n");

  struct dirent *dirent;
  int renamed = 0;

  while ((dirent = client_readdir(reader)) != NULL)
  {
    if (!strncmp(dirent->d_name, "renamed-", 8))
      renamed++;

    printf("> %s\n", dirent->d_name);
    free(dirent);
  }

  client_close(reader);

  printf("Directory shown.\n");

  printf("Renamed files: %d (%d)\n", renamed, expected);

  if (renamed != expected)
    return 1;

  return 0;
}

static size_t max(size_t a, size_t b)
{
  return a >= b ? a : b;
}

int main(int argc, char *argv[])
{
  if (argc < 3)
  {
    printf("Need a directory containing files and a new directory name.\n");
    return 1;
  }

  char *filename = argv[1], *newdir = argv[2];

  printf("Opening %s...\n", filename);

  show_directory(filename, 0);

  /* Rename some files. */

  size_t namelen = max(strlen(filename), strlen(newdir));
  char source[namelen + strlen("/renamed-XXXX.txt") + 10];
  char target[namelen + strlen("/renamed-XXXX.txt") + 10];
  int filenum, to_rename = 100;
  long err;

  for (filenum = 1; filenum <= to_rename; filenum++)
  {
    sprintf(source, "%s/file-%d.txt", filename, filenum);
    sprintf(target, "%s/renamed-%d.txt", filename, filenum);

    err = client_rename(source, target);

    if (err)
    {
      printf("Could not rename file %s: %s\n", source, l4sys_errtostr(err));
      return 1;
    }
  }

  /* Show the new listing. */

  if (show_directory(filename, to_rename))
    return 1;

  /* Create a directory and move files into it. */

  err = client_mkdir(newdir, 0755);

  if (err)
  {
    printf("Could not make directory: %s\n", newdir);
    return 1;
  }

  file_t *reader;
  struct dirent *dirent;

  reader = client_opendir(filename);

  if (!client_opened(reader))
  {
    client_close(reader);
    printf("Could not read from directory.\n");
    return 1;
  }

  printf("Renaming...\n");

  while ((dirent = client_readdir(reader)) != NULL)
  {
    if (!strncmp(dirent->d_name, "renamed-", 8))
    {
      sprintf(source, "%s/%s", filename, dirent->d_name);
      sprintf(target, "%s/%s", newdir, dirent->d_name);

      err = client_rename(source, target);

      if (err)
      {
        printf("Could not rename file %s: %s\n", source, l4sys_errtostr(err));
        return 1;
      }
    }

    free(dirent);
  }

  client_close(reader);

  /* Show the new directory's listing. */

  if (show_directory(newdir, to_rename))
    return 1;

  printf("End of test.\n");
  return 0;
}

// vim: tabstop=2 expandtab shiftwidth=2
